<?php


class RafflePlayWoo_OptionsManager {
    const COUNT_STARTS_AT       = 'ticket_count_starts_at';
    const LAST_TICKET_NO        = 'last_ticket_no';
    const LAST_ORDER_NO         = 'last_order_no';
    const STARTED_ORDERS        = 'started_orders';
    const DEFAULT_STARTS_AT     =  1;
    const LIVE_RAFFLE           = 'live_raffle';  
    const DEFAULT_DEBUG_NO      =  8888000; 
    const LAST_DEBUG_NO         = 'last_debug_no';
    const TICKET_PREFIX         = 'ticket_prefix'; 
    const START_DATE            = 'start_date';
    const START_TIME            = 'start_time';
    const END_DATE              = 'end_date';
    const END_TIME              = 'end_time';
    const EMAIL_HEADER          = 'email_header';
    const EMAIL_BODY            = 'email_body';
    const EMAIL_POS             = 'email_pos';
    const LIMIT_TICKETS         = 'limit_tickets';
    const LIMIT_TICKETS_NO      = 'limit_tickets_no';
    const FILTER_STATUS         = 'filter_status_view';
    const TABLES_CREATED        = 'tables_created'; 
    const RAFFLE_TERMINATE      = 'terminated';

    const DEFAULT_START_DATE    = '2010-01-01 00:00';
    const DEFAULT_END_DATE      = '2050-01-01 00:00'; 

    const NEW_VERSION           = 'new_version';
    const LAST_VERSION_CHECK    = 'last_version_check';

    const RAFFLE_NAME           = 'raffle_name';
    const INC_NAME              = 'inc_name';

    const SHORTCODE_MSG         = 'raffle'; 


    //-------------- license -----------------------------
    protected function get_newVersion(){
        return $this->getOption( self::NEW_VERSION, RAFFLE_PLAY_WOO_VERSION );
    }

    protected function set_newVersion( $value ){
        if( ! $this->updateOption( self::NEW_VERSION, $value ) )
            $this->addOption( self::NEW_VERSION, $value );
    }


    //-------------- version -----------------------------
    protected function get_lastVersionCheck(){
        $now_time = current_time( 'timestamp' );
        return $this->getOption( self::LAST_VERSION_CHECK, $now_time);
    }

    protected function set_lastVersionCheck( $value ){
        if( ! $this->updateOption( self::LAST_VERSION_CHECK, $value ) )
            $this->addOption( self::LAST_VERSION_CHECK, $value );
    }
    


    //-------------- raffle_name -----------------------------
    protected function get_raffle_name(){
        return stripslashes( esc_html__( $this->getOption( self::RAFFLE_NAME, 'Default') ) );
    }

    protected function set_raffle_name( $value ){
        if( ! $this->updateOption( self::RAFFLE_NAME, $value ) )
            $this->addOption( self::RAFFLE_NAME, $value );
    }

    //-------------- Include Raffle Name-----------------------------
    protected function get_inc_name(){
        return $this->getOption( self::INC_NAME, '0') == '1';
    }

    protected function set_inc_name( $value ){
        if( ! $this->updateOption( self::INC_NAME, $value ) )
            $this->addOption( self::INC_NAME, $value );
    }


    //================================================

    protected function updateLastTicketNo( $ticket_no, $isLiveRaffle ){

        if( $isLiveRaffle ){
            if( ! $this->updateOption( self::LAST_TICKET_NO, $ticket_no ) ){
                $this->addOption( self::LAST_TICKET_NO, $ticket_no );
            }
        }else{
            if( ! $this->updateOption( self::LAST_DEBUG_NO, $ticket_no ) ){
                $this->addOption( self::LAST_DEBUG_NO, $ticket_no );
            }
        }
    }

    protected function getDefaultDebugNo(){
        return self::DEFAULT_DEBUG_NO; 
    }

    protected function getLastTicketNo( $isLiveRaffle ){
        $ticket_start_at    = (int)$this->getOption( self::COUNT_STARTS_AT, self::DEFAULT_STARTS_AT );
        $last_ticket_no     = (int)$this->getOption( self::LAST_TICKET_NO, $ticket_start_at );              
        $last_debug_no      = (int)$this->getOption( self::LAST_DEBUG_NO, self::DEFAULT_DEBUG_NO );  

        if( $isLiveRaffle )
            return $last_ticket_no;
        else
            return $last_debug_no;        
    }

    protected function getTicketPrefix(){
        return ( $this->getOption( self::TICKET_PREFIX, 'Ticket-') );
    }

    protected function setTicketPrefix( $ticket_prefix ){
        if( ! $this->updateOption( self::TICKET_PREFIX, $ticket_prefix ) )
            $this->addOption( self::TICKET_PREFIX, $ticket_prefix );
    }

    protected function isTablesCreated(){
        return ( $this->getOption( self::TABLES_CREATED, 'yes') === 'yes' );
    }

    protected function setTablesCreated( $value ){
        if( ! $this->updateOption( self::TABLES_CREATED, $value ) )
            $this->addOption( self::TABLES_CREATED, $value );
    }

    protected function getFilterStatusView(){
        return ( $this->getOption( self::FILTER_STATUS, 'wc-processing,wc-completed' ) );
    }

    protected function setFilterStatusView( $new_status ){
        if( ! $this->updateOption( self::FILTER_STATUS, $new_status ) )
            $this->addOption( self::FILTER_STATUS, $new_status );
    }

    protected function isRaffleLimited(){
        return ( $this->getOption( self::LIMIT_TICKETS, 'no') === 'yes' );
    }

    protected function updateRaffleLimited ( $new_value ){
        if( ! $this->updateOption( self::LIMIT_TICKETS, $new_value ) )
            $this->addOption( self::LIMIT_TICKETS, $new_value );
    }


    protected function getEmailHeaderLbl(){
        return  stripslashes( esc_html( ( $this->getOption( self::EMAIL_HEADER, 'Raffle') ) ) );
    }
    
    protected function setEmailHeaderLbl( $email_header){
        if( ! $this->updateOption( self::EMAIL_HEADER, $email_header ) )
            $this->addOption( self::EMAIL_HEADER, $email_header );
    }

    protected function getEmailBodyLbl(){
        return stripslashes( esc_html( $this->getOption( self::EMAIL_BODY, 'Raffle Tickets') ) );
    }
    
    protected function setEmailBodyLbl( $email_body){
        if( ! $this->updateOption( self::EMAIL_BODY, $email_body ) )
            $this->addOption( self::EMAIL_BODY, $email_body );
    }

    protected function getEmailPos(){
        return ( $this->getOption( self::EMAIL_POS, '2') );
    }
    
    protected function setEmailPos( $email_pos ){
        if( ! $this->updateOption( self::EMAIL_POS, $email_pos ) )
            $this->addOption( self::EMAIL_POS , $email_pos );
    }

    protected function getStartDate(){
        return  ( trim( $this->getOption( self::START_DATE, '') ) );
    }

    protected function setStartDate( $start_date ){
        if( ! $this->updateOption( self::START_DATE, $start_date ) )
            $this->addOption( self::START_DATE, $start_date );
    }

    protected function getStartTime(){
        return ( trim( $this->getOption( self::START_TIME, '') ) );
    }

    protected function setStartTime( $start_time ){
        if( ! $this->updateOption( self::START_TIME, $start_time ) )
            $this->addOption( self::START_TIME, $start_time );
    }

    protected function getNowTime(){
        return current_time( 'timestamp');
    }


    protected function isRaffleTerminated(){
        return ( $this->getOption( self::RAFFLE_TERMINATE, 'no') === 'yes' );
    }

    protected function setRaffleTerminated( $new_value){
        if( ! $this->updateOption( self::RAFFLE_TERMINATE, $new_value ) )
            $this->addOption( self::RAFFLE_TERMINATE, $new_value );
    }

    protected function getLimitNo(){
        return  intval( ( $this->getOption( self::LIMIT_TICKETS_NO, '0') ) );
    }

    protected function setLimitNo( $limit_no ){
        if( ! $this->updateOption( self::LIMIT_TICKETS_NO, $limit_no ) )
            $this->addOption( self::LIMIT_TICKETS_NO, $limit_no );
    }

    protected function getEndDate(){
        return ( trim( $this->getOption( self::END_DATE, '') ) );
    }

    protected function setEndDate( $end_date ){
        if( ! $this->updateOption( self::END_DATE, $end_date ) )
            $this->addOption( self::END_DATE, $end_date );
    }

    protected function getEndTime(){
        return ( trim( $this->getOption( self::END_TIME, '') ) );
    }

    protected function setEndTime( $end_time ){
        if( ! $this->updateOption( self::END_TIME, $end_time ) )
            $this->addOption( self::END_TIME, $end_time );
    }

    protected function updateLastOrderId( $order_id ){
        if( ! $this->updateOption( self::LAST_ORDER_NO, $order_id ) )
            $this->addOption( self::LAST_ORDER_NO, $order_id );
    }

    protected function getLastOrderId( $default ){
        return (int)$this->getOption( self::LAST_ORDER_NO, $default );
    }

    protected function isLiveRaffle(){
        return ($this->getOption( self::LIVE_RAFFLE, '1' ) === '1'); 
    }

    protected function updateLiveRaffle( $value ){
        $return = true;
        if( ! $this->updateOption( self::LIVE_RAFFLE, $value ) )
            $return = $this->addOption( self::LIVE_RAFFLE, $value );

        return $return;  
    }

    protected function startedOrders(){
        if( ! $this->updateOption( self::STARTED_ORDERS, '1' ) )
            $this->addOption( self::STARTED_ORDERS, '1' ); 
    }

    protected function hasStartedOrders(){
        return ( $this->getOption( self::STARTED_ORDERS, '0' ) === '0' );
    }

    protected function getCountStartsAt(){
        return $this->getOption( self::COUNT_STARTS_AT, self::DEFAULT_STARTS_AT );
    }

    protected function updateCountStartsAt( $value ){
        $return = true;
        if( ! $this->updateOption( self::COUNT_STARTS_AT, $value ) )
            $return = $this->addOption( self::COUNT_STARTS_AT, $value );

        return $return;
    }

    public function getOptionNamePrefix() {
        return 'RafflePlayWoo_';
    }

    public function getOptionMetaData() {
        return array();
    }


    public function getOptionNames() {
        return array_keys($this->getOptionMetaData());
    }


    protected function initOptions() {
    }


    protected function deleteSavedOptions() {
        $optionMetaData = $this->getOptionMetaData();
        if (is_array($optionMetaData)) {
            foreach ($optionMetaData as $aOptionKey => $aOptionMeta) {
                $prefixedOptionName = $this->prefix($aOptionKey); // how it is stored in DB
                delete_option($prefixedOptionName);
            }
        }
    }


    public function getPluginDisplayName() {
        return get_class($this);
    }


    public function prefix($name) {
        $optionNamePrefix = $this->getOptionNamePrefix();
        if (strpos($name, $optionNamePrefix) === 0) { // 0 but not false
            return $name; // already prefixed
        }
        return $optionNamePrefix . $name;
    }



    public function getOption($optionName, $default = null) {
        $prefixedOptionName = $this->prefix($optionName); // how it is stored in DB       
        $retVal = get_option($prefixedOptionName);
        if (!$retVal && $default) {
            $retVal = $default;
        }
        return $retVal;
    }


    public function deleteOption($optionName) {
        $prefixedOptionName = $this->prefix($optionName); // how it is stored in DB
        return delete_option($prefixedOptionName);
    }


    public function addOption($optionName, $value) {
        $prefixedOptionName = $this->prefix($optionName); // how it is stored in DB
        return add_option($prefixedOptionName, $value);
    }

    public function updateOption($optionName, $value) {
        $prefixedOptionName = $this->prefix($optionName); // how it is stored in DB
        return update_option($prefixedOptionName, $value);
    }

    public function getRoleOption($optionName) {
        $roleAllowed = $this->getOption($optionName);
        if (!$roleAllowed || $roleAllowed == '') {
            $roleAllowed = 'Administrator';
        }
        return $roleAllowed;
    }

    public function isUserRoleEqualOrBetterThan($roleName) {
        if ('Anyone' == $roleName) {
            return true;
        }
        $capability = $this->roleToCapability($roleName);
        return current_user_can($capability);
    }

    public function canUserDoRoleOption($optionName) {
        $roleAllowed = $this->getRoleOption($optionName);
        if ('Anyone' == $roleAllowed) {
            return true;
        }
        return $this->isUserRoleEqualOrBetterThan($roleAllowed);
    }

    public function registerSettings() {
        $settingsGroup = get_class($this) . '-settings-group';
        $optionMetaData = $this->getOptionMetaData();
        foreach ($optionMetaData as $aOptionKey => $aOptionMeta) {
            register_setting($settingsGroup, $aOptionMeta);
        }
    }
 

    protected function getMySqlVersion() {
        global $wpdb;
        $rows = $wpdb->get_results('select version() as mysqlversion');
        if (!empty($rows)) {
             return $rows[0]->mysqlversion;
        }
        return false;
    }

 
    public function getEmailDomain() {
        // Get the site domain and get rid of www.
        $sitename = strtolower($_SERVER['SERVER_NAME']);
        if (substr($sitename, 0, 4) == 'www.') {
            $sitename = substr($sitename, 4);
        }
        return $sitename;
    }

}