<?php

include_once( 'RafflePlayWoo_LifeCycle.php');
include_once( 'd/D_RafflePlayWoo_Main.php');

class RafflePlayWoo_Plugin extends RafflePlayWoo_LifeCycle{
    
    public function getPluginDisplayName(){
        return 'Raffle Play Woo';
    }

    protected function getMainPluginFileName(){
        return 'raffle-play-woo.php';
    }

    private function createMainTables( $table_name, $charset_collate ){

        $backup_field = "backup_name varchar(30) default '',";
        if( $table_name !== $this->prefixTableName('main_bk') )
            $backup_field = "";

        $sql_query =  "CREATE TABLE IF NOT EXISTS {$table_name} (
                    id mediumint(5) NOT NULL auto_increment,
                    order_id int(10) NOT NULL default '0',                            
                    customer_id int(5) NOT NULL default '0',
                    product_id  int(5) NOT NULL default '0',
                    ticket int(10) NOT NULL default '0',
                    quantity int(5) NOT NULL default '0',                                   
                    tickets_purchased int(5) NOT NULL default '0',
                    catid_one int(5) NOT NULL default '0',
                    catid_two int(5) NOT NULL default '0',
                    catid_three int(5) NOT NUll default '0',
                    ticket_no int(10) NOT NULL default '0',
                    ticket_from int(10) NOT NULL default '0',
                    ticket_to int(10) NOT NULL default '0',                            
                    ticket_txt text NOT NULL default '',
                    total_price float NOT NULL default '0.0', 
                    live_order int(1) NOT NULL default '0',
                    deleted int(1) NOT NULL default '0', 
                    order_status varchar(30) default '',
                    {$backup_field}                   
                    created_at datetime NOT NULL default CURRENT_TIMESTAMP,                                 
                    PRIMARY KEY  (id),
                    KEY order_id (order_id),                            
                    KEY customer_id (customer_id),
                    KEY product_id (product_id),
                    KEY ticket (ticket),
                    KEY ticket_no (ticket_no),
                    KEY catid_one (catid_one),
                    KEY catid_two (catid_two),
                    KEY order_status (order_status),
                    KEY created_at (created_at)  
                ) $charset_collate;";  

        return $sql_query;
    }

    private function createAuditTable( $table_name, $charset_collate ){
        $sql_query =  "CREATE TABLE IF NOT EXISTS {$table_name} (
                    id mediumint(5) NOT NULL auto_increment,
                    order_id int(5) NOT NULL default '0',
                    action_type varchar(30) NOT NULL default '',
                    product_id int(5) NOT NULL default '0',
                    customer_id int(5) NOT NULL default '0',
                    old_value varchar(30) NOT NULL default '',
                    new_value varchar(30) NOT NULL default '',
                    action_txt  varchar(200) NOT NULL default '',
                    created_at datetime NOT NULL default CURRENT_TIMESTAMP, 
                    created_by int(5) NOT NULL default '0',                                
                    PRIMARY KEY  (id),
                    KEY order_id (order_id),
                    KEY product_id (product_id),
                    KEY customer_id (customer_id),
                    KEY action_type (action_type)                   
                ) $charset_collate;"; 
        return $sql_query;
    }

    private function createRaffleTable( $table_name, $charset_collate ){
        $sql_query =  "CREATE TABLE IF NOT EXISTS {$table_name} (
                    raffle_id mediumint(5) NOT NULL auto_increment,
                    r_name varchar(50) NOT NULL default '',
                    is_live int(1) NOT NULL default '1',
                    is_terminated int(1) NOT NULL default '0',
                    start_ticket int(10) NOT NULL default '1000',
                    last_purchased int(10) NOT NULL default '0',
                    is_limit int(1) NOT NULL default '0',
                    limit_no int(10) NOT NULL default '0',
                    prefix varchar(20) NOT NULL default '',
                    start_datetime int NOT NULL default '0',
                    end_datetime int NOT NULL default '0',
                    extra_email text NOT NULL default '',
                    extra_email_in int(1) NOT NULL default '0',
                    deleted int(1) NOT NULL default '0',
                    shortcode varchar(50) NOT NULL default '',
                    shrt_extra varchar(150) NOT NULL default '',
                    shrt_ckb  int(1) NOT NULL default '0',
                    field_logic int(1) NOT NULL default '0',
                    field_int int(10) NOT NULL default '0',
                    field_char varchar(100) NOT NULL default '',                             
                    PRIMARY KEY  (raffle_id),
                    KEY is_live (is_live)                
                ) $charset_collate;"; 

        return $sql_query;

    }

    private function createWinnersTable( $table_name, $charset_collate ){
        $sql_query =  "CREATE TABLE IF NOT EXISTS {$table_name} (
            id mediumint(5) NOT NULL auto_increment,
            ticket int(12) NOT NULL default '0',
            raffle_id int(5) NOT NULL default '0', 
            prize varchar(50) NOT NULL default '',
            extra_text varchar(200) NOT NULL default '',
            extra_text_two varchar(200) NOT NULL default '',
            extra_ckb int(1) NOT NULL default '0',  
            deleted int(1) NOT NULL default '0',                 
            PRIMARY KEY  (id),
            KEY ticket (ticket),
            KEY raffle_id (raffle_id)                
        ) $charset_collate;"; 

        return $sql_query;
    }

    private function createRaffleProductTableJoin( $table_name, $charset_collate ){
        $sql_query =  "CREATE TABLE IF NOT EXISTS {$table_name} (
            id mediumint(5) NOT NULL auto_increment,
            product_id int(5) NOT NULL default '0',
            raffle_id int(5) NOT NULL default '0',                     
            PRIMARY KEY  (id),
            KEY product_id (product_id),
            KEY raffle_id (raffle_id)                
        ) $charset_collate;"; 

        return $sql_query;
    } 
    

    protected function installDatabaseTables(){
        global $wpdb;
       
        $table_main         = $this->prefixTableName('main');
        $table_main_b       = $this->prefixTableName('main_bk');
        $table_audit        = $this->prefixTableName('audit');  
        $table_r_product    = $this->prefixTableName('raffle');     
        $table_r_p_join     = $this->prefixTableName('product');
        $table_winners      = $this->prefixTableName('winners');

        $charset_collate    = $wpdb->get_charset_collate();
          
        $sql_main           = $this->createMainTables( $table_main, $charset_collate );

        $sql_main_b         = $this->createMainTables( $table_main_b, $charset_collate );

        $sql_audit          = $this->createAuditTable( $table_audit, $charset_collate );   

        $sql_raffle         = $this->createRaffleTable( $table_r_product, $charset_collate );

        $sql_r_p_join       = $this->createRaffleProductTableJoin( $table_r_p_join, $charset_collate );

        $sql_winners        = $this->createWinnersTable( $table_winners, $charset_collate );

        require_once( ABSPATH . 'wp-admin/includes/upgrade.php' );

        dbDelta( $sql_main );  
        dbDelta( $sql_main_b ) ;    
        dbDelta( $sql_audit  ); 
        dbDelta( $sql_raffle );
        dbDelta( $sql_r_p_join );
        dbDelta( $sql_winners );
        
        //check if tables were installed
        $db_obj = new RafflePlayWoo_MainTbl();
        $tables_created = $db_obj->checkDbHealth();
        $this->setTablesCreated( $tables_created );      
    }

    private function drp_local_txt(){
        //here goes all the localization to front-end
        return array(
            'saved'                 => __('Saved', RAFFLE_PLAY_WOO_DOMAIN ),
            'error_parsing_data'    => __ ('Error Parsing Data', RAFFLE_PLAY_WOO_DOMAIN),
            'order_id'              => __('Order Id', RAFFLE_PLAY_WOO_DOMAIN ),
            'ticket'                => __('Ticket', RAFFLE_PLAY_WOO_DOMAIN),
            'ticket_from'           => __('Ticket From', RAFFLE_PLAY_WOO_DOMAIN),
            'ticket_to'             => __('Ticket To', RAFFLE_PLAY_WOO_DOMAIN),
            'no_tickets'            => __('No Tickets', RAFFLE_PLAY_WOO_DOMAIN),
            'total_tickets'         => __('Total Tickets', RAFFLE_PLAY_WOO_DOMAIN),
            'order_sale'            => __('Order Sale', RAFFLE_PLAY_WOO_DOMAIN),
            'first_name'            => __('First Name', RAFFLE_PLAY_WOO_DOMAIN),
            'last_name'             => __('Last Name', RAFFLE_PLAY_WOO_DOMAIN),
            'email'                 => __('Email', RAFFLE_PLAY_WOO_DOMAIN),
            'customer_id'           => __('Customer Id', RAFFLE_PLAY_WOO_DOMAIN),
            'order_date_time'       => __('Order Date/Time', RAFFLE_PLAY_WOO_DOMAIN),
            'raffle_ticket'         => __('Raffle Ticket', RAFFLE_PLAY_WOO_DOMAIN),
            'invalid_number'        => __('Invalid Number', RAFFLE_PLAY_WOO_DOMAIN),
            'greater_than_zero'     => __('Should be greater than zero', RAFFLE_PLAY_WOO_DOMAIN ),
            'country'               => __('Country', RAFFLE_PLAY_WOO_DOMAIN ),
            'county'                => __('County', RAFFLE_PLAY_WOO_DOMAIN ),
            'phone'                 => __('Phone', RAFFLE_PLAY_WOO_DOMAIN ),
            'city'                  => __('City', RAFFLE_PLAY_WOO_DOMAIN ),
            'id'                    => __('Id', RAFFLE_PLAY_WOO_DOMAIN ),
            'raffle_name'           => __('Raffle Name', RAFFLE_PLAY_WOO_DOMAIN ),
            'live'                  => __('Live', RAFFLE_PLAY_WOO_DOMAIN ),
            'terminated'            => __('Terminated', RAFFLE_PLAY_WOO_DOMAIN ),
            'yes'                   => __('Yes', RAFFLE_PLAY_WOO_DOMAIN ),
            'no'                    => __('No', RAFFLE_PLAY_WOO_DOMAIN ),
            'products'              => __('Products', RAFFLE_PLAY_WOO_DOMAIN ),
            'product'               => __('Product', RAFFLE_PLAY_WOO_DOMAIN ),
            'limited'               => __('Limited', RAFFLE_PLAY_WOO_DOMAIN ),
            'limit_no'              => __('Limit No', RAFFLE_PLAY_WOO_DOMAIN ),
            'info'                  => __('Info', RAFFLE_PLAY_WOO_DOMAIN ),
            'start_ticket'          => __('Start Ticket', RAFFLE_PLAY_WOO_DOMAIN ),
            'new_raffle'            => __('New Raffle', RAFFLE_PLAY_WOO_DOMAIN ),
            'edit_raffle'           => __('Edit Raffle', RAFFLE_PLAY_WOO_DOMAIN ),
            'no_row_selected'       => __('No Row Selected', RAFFLE_PLAY_WOO_DOMAIN ),
            'no_raffle_selected'    => __('No Raffle Selected', RAFFLE_PLAY_WOO_DOMAIN ),
            'raffle_name_required'  => __('Raffle Name Required', RAFFLE_PLAY_WOO_DOMAIN ),
            'invalid_start_ticket'  => __('Invalid Start Ticket Number', RAFFLE_PLAY_WOO_DOMAIN),
            'invalid_limit_no'      => __('Invalid Limit Number', RAFFLE_PLAY_WOO_DOMAIN ),
            'start_date'            => __('Start Date', RAFFLE_PLAY_WOO_DOMAIN ),
            'end_date'              => __('End Date', RAFFLE_PLAY_WOO_DOMAIN ),
            'prefix'                => __('Prefix', RAFFLE_PLAY_WOO_DOMAIN ),
            'test'                  => __('Test', RAFFLE_PLAY_WOO_DOMAIN ),
            'last_ticket'           => __('Last Ticket', RAFFLE_PLAY_WOO_DOMAIN ),
            'sold_tickets'          => __('Sold Tickets', RAFFLE_PLAY_WOO_DOMAIN ),
            'shortcode'             => __('Shortcode', RAFFLE_PLAY_WOO_DOMAIN),
            'address'               => __('Address', RAFFLE_PLAY_WOO_DOMAIN),
            'extra_info'            => __('Extra Info', RAFFLE_PLAY_WOO_DOMAIN),
            'invalid_ticket_no'     => __('Invalid Ticket Number', RAFFLE_PLAY_WOO_DOMAIN),
            'prize'                 => __('Prize', RAFFLE_PLAY_WOO_DOMAIN ),
            'delete_row_confirm'    => __('Are you sure you want to delete it?', RAFFLE_PLAY_WOO_DOMAIN),
            'shortcode_info'        => __("To display the raffle name add show_raffle_name in the shortcode", RAFFLE_PLAY_WOO_DOMAIN),
            'raffle'                => __("Raffle", RAFFLE_PLAY_WOO_DOMAIN),
            'delete_raffle_tkts'    => __("Are you sure you want to delete this raffle's tickets?", RAFFLE_PLAY_WOO_DOMAIN),
            'records_deleted'       => __("Records Deleted", RAFFLE_PLAY_WOO_DOMAIN ),
            'duplicated_meta_name'  => __("Warning. There is another column with same DB Meta Name", RAFFLE_PLAY_WOO_DOMAIN )
        );

    }

    protected function uninstallDatabaseTables(){
    }

    public function upgrade(){
    }


    public function addActionsAndFilters(){    
  
        add_action('admin_menu', array(&$this, 'addMenuPages'));     
        add_action('admin_enqueue_scripts', array( &$this, 'pw_load_scripts')); 
 
        add_action( 'woocommerce_product_options_general_product_data', array( &$this, 'drp_woo_product_checkbox' ) );
        add_action( 'woocommerce_process_product_meta', array( &$this, 'drp_woo_save_custom_fields' ), 10, 1);
        
        add_action( 'woocommerce_admin_order_data_after_order_details', array( &$this, 'drp_display_order_data_in_admin'), 20, 1 ); //admin order view
        add_filter( 'manage_edit-product_columns', array( &$this, 'drp_add_product_column' ), 10, 1 );
        add_action( 'manage_product_posts_custom_column', array( &$this, 'drp_product_column_raffle'), 11, 2 );
        add_filter( 'plugin_row_meta',  array( &$this, 'drp_woo_plugin_links'), 10, 2 ); 
        
        add_action( 'woocommerce_email_after_order_table', array( &$this, 'drp_order_email_instructions'), 21, 4); 
     
        
        /* order status */
        
        add_action( 'woocommerce_order_status_processing', array( &$this, 'drp_woo_process_checkout' ), 10, 1);
        add_action( 'woocommerce_order_status_completed', array( &$this, 'drp_woo_process_checkout' ), 10, 1);   

        if( is_admin() ){
                add_action( 'woocommerce_order_status_failed', array( &$this, 'drp_woo_mark_deleted_order' ), 10, 1); 
                add_action( 'woocommerce_order_status_refunded', array( &$this, 'drp_woo_mark_deleted_order' ), 10, 1); 
                add_action( 'woocommerce_order_status_cancelled', array( &$this, 'drp_woo_mark_deleted_order' ), 10, 1);             
        }

        // display the extra data on order received page and my-account order review     
        add_action( 'woocommerce_thankyou', array( &$this,'drp_display_order_data_thank_you'), 20 );
        add_action( 'woocommerce_view_order', array( &$this, 'drp_display_order_data'), 20 );     

        add_action('wp_ajax_drp_save_settings', array( &$this, 'drp_save_settings') );
        add_action('wp_ajax_drp_get_data', array(&$this, 'drp_get_data') );
        add_action('wp_ajax_drp_fix_db', array(&$this, 'drp_fix_db') );         
               
    }


    public function pw_load_scripts( $hook ){
        global $rpwoo_main_page;        
        global $rpwoo_view_page;
        global $rpwoo_audit_page;
        global $rpwoo_reports_page;
        global $rpwoo_info_page;
        global $rpwoo_product_page;
        global $rpwoo_winners_page;

        if( ($rpwoo_main_page != $hook) && ($rpwoo_view_page != $hook ) && 
            ($rpwoo_audit_page != $hook) && ( $rpwoo_reports_page != $hook ) &&
            ($rpwoo_info_page != $hook) && ( $rpwoo_product_page != $hook ) &&
            ($rpwoo_winners_page != $hook ) )
            return;

        wp_enqueue_style('rpwoo-style-main', RAFFLE_PLAY_WOO_URL.'/css/drp_main.css' . '?v='.$this->getVersion() );
        wp_enqueue_style('rpwoo-style-bootstrap', RAFFLE_PLAY_WOO_URL.'/css/bootstrap.min.css' . '?v='.$this->getVersion() );
        wp_enqueue_style('rpwoo-style-fontawsome', RAFFLE_PLAY_WOO_URL.'/css/fa/css/all.css'.'?v='.$this->getVersion() );
        wp_enqueue_style('rpwoo-style-bootstrap-toggle', RAFFLE_PLAY_WOO_URL.'/css/bootstrap-toggle.min.css'. '?v='.$this->getVersion() );     
        wp_enqueue_style('rpwoo-style-bootstrap-table', RAFFLE_PLAY_WOO_URL.'/css/bootstrap-table.css' . '?v='.$this->getVersion() );
        wp_enqueue_style('rpwoo-my-style-clockpicker', RAFFLE_PLAY_WOO_URL.'/css/clockpicker.css' . '?v='.$this->getVersion() );
        wp_enqueue_style('rpwoo_my-style-jquery-theme',   RAFFLE_PLAY_WOO_URL.'/jquery/themes/sunny/jquery-ui.min.css' . '?v='.$this->getVersion() );

        
        wp_enqueue_script('rpwoo-script-bootstrap-toggle', RAFFLE_PLAY_WOO_URL.'/js/bootstrap-toggle.min.js' . '?v='.$this->getVersion() );
        wp_enqueue_script('rpwoo-script-bootstrap', RAFFLE_PLAY_WOO_URL.'/js/bootstrap.min.js' . '?v='.$this->getVersion() );        
        wp_enqueue_script('rpwoo-script-messages',  RAFFLE_PLAY_WOO_URL.'/js/rpwoo_script_message.js'.'?v=' . $this->getVersion() );     
        wp_enqueue_script('rpwoo-script-drp-bootstrap-table', RAFFLE_PLAY_WOO_URL.'/js/bootstrap-table.js'.'?v=' . $this->getVersion() ); 
        wp_enqueue_script('rpwoo_plugin_view_clockpicker', RAFFLE_PLAY_WOO_URL.'/js/clockpicker.js' . '?v='.$this->getVersion() ); 
       
        wp_enqueue_script( 'jquery-ui-datepicker' ); 

        //localization in the front end 
        wp_localize_script('rpwoo-script-messages', 'rpwoo_local_lng', $this->drp_local_txt() );

        if( $rpwoo_main_page == $hook ){
            wp_enqueue_script('rpwoo_plugin_main_script', RAFFLE_PLAY_WOO_URL.'/js/drp_main.js' . '?v='.$this->getVersion() );
        }else if( $rpwoo_product_page == $hook ){
            wp_enqueue_script('rpwoo_plugin_view', RAFFLE_PLAY_WOO_URL.'/js/drp_raffles.js' . '?v='.$this->getVersion() ); 
        }else if( $rpwoo_view_page == $hook ){           
            wp_enqueue_script('rpwoo_plugin_view', RAFFLE_PLAY_WOO_URL.'/js/drp_view.js' . '?v='.$this->getVersion() );  
        }else if( $rpwoo_audit_page == $hook ){
            wp_enqueue_script('rpwoo_plugin_audit', RAFFLE_PLAY_WOO_URL.'/js/drp_audit.js' . '?v='.$this->getVersion() ); 
        }else if( $rpwoo_reports_page == $hook ){
            wp_enqueue_style('rpwoo-style-reports_page', RAFFLE_PLAY_WOO_URL.'/css/drp_reports.css' . '?v='.$this->getVersion() );
            wp_enqueue_script('rpwoo_plugin_reports_script', RAFFLE_PLAY_WOO_URL.'/js/drp_reports.js' . '?v='.$this->getVersion() );
        }else if( $rpwoo_winners_page == $hook ){
            wp_enqueue_script('rpwoo_plugin_view', RAFFLE_PLAY_WOO_URL.'/js/drp_winners.js' . '?v='.$this->getVersion() );   
        }        
           
    }
}